//
//  GIF to BMP Converter
//
//   Copyright (c) 2005 Colosseum Builders, Inc.
//   www.colosseumbuilders.com
//
//  Author: John M. Miano
//  Date:    June 28, A.D. 2005
//
//  Usage:
//
//     giftobmp [flags] input output
//
//    input => Input file name, defaults to .GIF extention if none specified.
//    output => Output file name, defaults to .BMP extension if none specified
//
//    flags
//      -v  Verbose, prints file structure information
//      -m Multi-image mode - Reads all images within the GIF file. It creates
//           files output.bmp, output_1.bmp, output_2.bmp ... output_n.bmp
//
//
#include <cstdlib>
#include <sysutils.hpp>
#include "gifdecoder.h"
#include "bmpencoder.h"

using namespace Colosseum ;
using namespace std ;

void Usage (int argc, char *argv [])
{
  cerr << "Usage: '" << argv [0] << "' [-flags-] input-png-file output-jpg-filename" << endl ;
  cerr << "    -v     Verbose" << endl ;
  cerr << "    -m     Multi-Image Mode" << endl ;
  exit (1) ;
}

void DecodeProgress (BitmapImageCoder &coder, 
                     void *data,
                     unsigned int currentpass,
                     unsigned int passcount,
                     unsigned int progress,
                     bool &cancel) 
{
    cout << currentpass << '/' << passcount <<  ' ' <<  progress << "%                            " << '\r' ;
    return ;
}

int main (int argc, char *argv [])
{
  BitmapImage image ;
  BmpEncoder bmp ;
  GifDecoder gif ;
  bool verbose = false ;
  bool multiimagemode = false ;

  if (argc < 3)
    Usage (argc, argv) ;

  for (unsigned int ii = 1 ; ii < argc - 2 ; ++ ii)
  {
    if (argv [ii][0] != '-')
      Usage (argc, argv) ;
    switch (argv [ii][1])
    {
    case 'v': verbose = true ; break ;
    case 'm': multiimagemode = true ; break ;
    default: Usage (argc, argv) ;
    }
  }


  AnsiString input = argv [argc - 2] ;
  AnsiString output = argv [argc - 1] ;

  if (ExtractFileExt (input) == "")
    input = input + ".gif" ;

  AnsiString basename = ExtractFileName (output) ;
  if (ExtractFileExt (output) == "")
    output = output + ".bmp" ;

  gif.setVerbose (verbose) ;
  gif.setMultiImageMode (multiimagemode) ;
  if (! verbose)
  {
      bmp.setProgressFunction (DecodeProgress, 0) ;
      gif.setProgressFunction (DecodeProgress, 0) ;
  }
  cout << "GIF Decode" << endl ;
  gif.readImageFile (input.c_str (), image) ;
  cout << endl ;
  cout << "BMP Encode" << endl ;
  bmp.writeImageFile (output.c_str (), image) ;

  // Possibly read the remaining images in the file.
  for (unsigned int filecount = 1 ;
       gif.moreImages () && gif.getMultiImageMode () ; ++ filecount)
  {
    output = basename + '_' + IntToStr (filecount) + ".bmp" ;
    cout << "GIF Decode" << endl ;
    gif.readNextImage (image) ;
    cout << "BMP Encode" << endl ;
    bmp.writeImageFile (output.c_str (), image) ;
  }
  return 0 ;
}
