#ifndef CB__GIF_H
#define CB__GIF_H
//
// Copyright (c) 1997,1998 Colosseum Builders, Inc.
// All rights reserved.
//
// Colosseum Builders, Inc. makes no warranty, expressed or implied
// with regards to this software. It is provided as is.
//
// See the README.TXT file that came with this software for restrictions
// on the use and redistribution of this file or send E-mail to
// info@colosseumbuilders.com
//

//
//  Title:  GIF private definitions
//
//  Author:  John M. Miano  miano@colosseumbuilders.com
//

#include <iostream>

#include "systemspecific.h"

const int GifMaxBitsPerCode = 12 ; // Up to 12 bits can be used in compresson
// Largest number of data bytes in a data block.
const unsigned int GifInputBufferSize = 255 ;
const int GifMaxColors = 256 ;

#include "beginpackbyte.h"
struct GifHeader
{
  BYTE1 file_signature [3] ;
  BYTE1 file_version [3] ;
  void print (std::ostream &strm = std::cout) const
  {
    using namespace std ;
    strm << "{ GIF Header" << endl ;
    strm << "  Signature: " << file_signature [0] << file_signature [1] << file_signature [2] << endl ;
    strm << "  Version: " << file_version [0] << file_version [1] << endl ;
    strm << "}" << endl ;
  }
} ;
#include "endpackbyte.h"

#include "beginpackbyte.h"
struct GifScreenDescriptor
{
  UBYTE2 logical_screen_width ;
  UBYTE2 logical_screen_height ;
  UBYTE1 flags ;
  int globalColorTableSize () const { return (flags & 0x7) ; }
  void globalColorTableSize (int value) { flags &= ~0x7 ; flags |= (value & 0x7) ; }
  bool sortFlag () const { return (flags & 0x8) != 0 ; }
  void sortFlag (bool value) { if (value) flags |= 0x8 ; else flags &= ~ 0x8 ; }
  int colorResolution () const { return (flags >> 4) & 0x7 ; }
  void colorResolution (int value) { flags &= ~0x70 ; flags |= ((value & 0x7) << 4) ; }
  bool globalColorTableFlag () const { return (flags & 0x80) != 0 ; }
  void globalColorTableFlag (bool value) { if (value) flags |= 0x80 ; else flags &= ~0x80 ; }
  UBYTE1 background_color_index ;
  UBYTE1 pixel_aspect_ratio ;

  void print (std::ostream &strm = std::cout) const
  {
    using namespace std ;
    strm << "{ Screen Descriptor" << endl ;
    strm << "        Logical Screen Width: " << logical_screen_width << endl ;
    strm << "       Logical Screen Height: " << logical_screen_height << endl ;
    strm << "  Size of Global Color Table: " << globalColorTableSize () << endl ;
    strm << "                   Sort Flag: " << (sortFlag () ? "True" : "False") << endl ;
    strm << "            Color Resolution: " << colorResolution () << endl ;
    strm << "     Global Color Table Flag: " << (sortFlag () ? "True" : "False") << endl ;
    strm << "}" << endl ;
    return ;
  }
} ;
#include "endpackbyte.h"

#include "beginpackbyte.h"
struct GifImageDescriptor
{
  UBYTE2 left_position ;
  UBYTE2 top_position ;
  UBYTE2 image_width ;
  UBYTE2 image_height ;
  UBYTE1 flags ;
  int colorTableSize () const { return flags & 0x7 ; }
  void colorTableSize (int value) { flags &= ~0x7 ; flags |= (flags & 0x7) ; }
  bool sortFlag () const { return (flags & 0x20) != 0 ; }
  void sortFlag (bool value) { if (value) flags |= 0x20 ; else flags &= ~0x20 ; }
  bool interlaceFlag () const { return (flags & 0x40) != 0 ; }
  void interlaceFlag (bool value) { if (value) flags |=  0x40 ; else flags &= ~0x40 ; }
  bool localColorTableFlag () const { return (flags & 0x80) != 0 ; }
  void localColorTableFlag (bool value) { if (value) flags |= 0x80 ; else flags &= ~0x80 ; }

  void print (std::ostream &strm = std::cout) const
  {
    using namespace std ;
    strm << "{ GIF Image Descriptor" << endl ;
    strm << "          Left Position: " << left_position << endl ;
    strm << "           Top Position: " << top_position << endl ;
    strm << "                  Width: " << image_width << endl ;
    strm << "                 Height: " << image_height << endl ;
    strm << " Local Color Table Flag: " << (localColorTableFlag () ? "True" : "False") << endl ;
    strm << "         Interlace Flag: " << (interlaceFlag () ? "True" : "False") << endl ;
    strm << "              Sort Flag: " << (sortFlag () ? "True" : "False") << endl ;
    strm << "       Color Table Size: " << colorTableSize () << endl ;
    strm << "}" << endl ;
    return ;
  }
} ;
#include "endpackbyte.h"

#include "beginpackbyte.h"
struct GifColorTableEntry
{
  UBYTE1 red ;
  UBYTE1 green ;
  UBYTE1 blue ;
} ;
#include "endpackbyte.h"

#include "beginpackbyte.h"
struct GifGraphicControlBlock
{
  UBYTE1 block_size ;
  UBYTE1 flags ;
  bool transparentColorFlag () const { return (flags & 0x1) != 0 ; }
  void transparentColorFlag (bool value) { if (value) flags |= 0x1 ; else flags &= ~ 0x1 ; }
  bool userInputFlag () const { return (flags & 0x2) != 0 ; }
  void userInputFlag (bool value) { if (value) flags |= 0x2 ; else flags &= ~ 0x2 ; }
  int disposalMethod () const { return (flags >> 2) & 0x7 ; }
  void disposalMethod (int value) { flags &= ~ 0x1C ; flags |= ((value & 0x7) << 2) ; }
  UBYTE2 delay_time ;
  UBYTE1 transparent_color ;

  void print (std::ostream &strm = std::cout) const
  {
    using namespace std ;
    strm << "{ Graphic Control Block " << endl ;
    cout << "    Delay Time: " << delay_time << endl ;
    if (transparentColorFlag ())
    {
       cout << "    Transparent Color Flag: "
            << (transparentColorFlag () ? "True" : "False") << endl ;
    }
    if (userInputFlag ())
      cout << "    User Input: True" << endl ;
    else
      cout << "    User Input: False" << endl ;

    cout << "    Disposal Method:  " << disposalMethod () << endl ;
    cout << "}" << endl ;
  }
} ;
#include "endpackbyte.h"

#include "beginpackbyte.h"
struct GifApplicationBlock
{
  UBYTE1 block_size ;
  UBYTE1 application_name [8] ;
  UBYTE1 authentication_code [3] ;

  void print (std::ostream &strm = std::cout) const
  {
    using namespace std ;
    strm << "{ Application Extension" << endl ;
//####    strm << "    Application Name: " <<  string (application_name, 0, sizeof (application_name))  << endl ;
    strm << "      Application ID: " <<  static_cast<unsigned int>(authentication_code [0]) <<  ' ' <<
                        static_cast<unsigned int>(authentication_code [1]) <<  ' ' <<
                        static_cast<unsigned int>(authentication_code [2]) << endl ;
    strm << "}" << endl  ;
    return ;
  }
} ;
#include "endpackbyte.h"

#include "beginpackbyte.h"
struct GifPlainTextBlock
{
  UBYTE1 block_size ;
  UBYTE2 text_left ;
  UBYTE2 text_top ;
  UBYTE2 text_width ;
  UBYTE2 text_height ;
  UBYTE1 char_width ;
  UBYTE1 char_height ;
  UBYTE1 foreground_color ;
  UBYTE1 background_color ;
  void print (std::ostream &strm = std::cout) const
  {
    using namespace std ;
    strm << "{ GifPlainTextBlock" << endl ;
    strm << "         Text Left: " << text_left << endl ;
    strm << "          Text Top: " << text_top << endl ;
    strm << "        Text Width: " << text_width << endl ;
    strm << "       Text Height: " << text_height << endl ;
    strm << "   Character Width: " << static_cast<unsigned int>(char_width) << endl ;
    strm << "   Character Width: " << static_cast<unsigned int>(char_width) << endl ;
    strm << "  Foreground Color: " << static_cast<unsigned int>(foreground_color) << endl ;
    strm << "  Background Color: " << static_cast<unsigned int>(background_color) << endl ;
    strm << "}" << endl ;
  }
} ;
#include "endpackbyte.h"

enum GifBlockType
{
  GifTerminator = 0x0,
  GifImageSeparator = 0x2C,
  GifExtension = 0x21,
  GifCommentExtension = 0xFE,
  GifPlainTextExtension = 0x01,
  GifApplicationExtension = 0xFF,
  GifGraphicControlExtension=0xF9,
  GifTrailer = 0x3B,
} ;

#endif
