#include <cerrno>
#include <cstring>
#include <fstream>
#include <iostream>
#include "gifinputfilestream.h"
#include "bitmapimage.h"

using namespace std ;

namespace Colosseum
{
GifInputFileStream::GifInputFileStream (unsigned int buffersize)
: input_buffer (buffersize == 0 ? DEFAULTSIZE : buffersize) 
, end_reached (false) 
{
}

GifInputFileStream::~GifInputFileStream ()
{
  close () ;
}


void GifInputFileStream::open (const std::string &filename) 
{
  close () ;

  input_stream.close () ;
  input_stream.clear () ;
  input_stream.open (filename.c_str (), ios::binary) ;
  if (! input_stream)
  {
    string msg = "GIF: Cannot open input file '" ;
    msg += filename + "'\n" + LastMessage () ;
    throw GifInputStreamError (msg) ;
  }
  end_reached = false ;
  return ;
}

InputByteStream::POSITIONTYPE GifInputFileStream::tellg () 
{
  return input_stream.tellg () ;
}


void GifInputFileStream::seekg (InputByteStream::POSITIONTYPE position) 
{
  exitBitMode () ;
  input_stream.seekg (position) ;
}

void GifInputFileStream::close ()
{
  if (input_stream)
    input_stream.close () ;
}

void GifInputFileStream::fillBuffer ()
{
  input_stream.read (&input_buffer [0], input_buffer.getSize ()) ;
  unsigned int count = input_stream.gcount () ;
  if (count == 0)
  {
    current_byte = 0 ;
    buffer_limit = 0 ; 
  }
  else
  {
    current_byte = &input_buffer [0] ;
    buffer_limit = &input_buffer [count] ;
  }
  if  (count != input_buffer.getSize ())
    end_reached = true ;
}

bool GifInputFileStream::endReached ()
{
  return end_reached ;
}
}